@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2005-2007 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2006 David Zeuthen <davidz@redhat.com>
  *
  * Licensed under the GNU General Public License Version 2
@@ -154,7 +154,7 @@ gpm_brightness_kbd_set_hw (GpmBrightnessKbd *brightness,
 	gpm_debug ("Setting %i of %i", brightness_level_hw, brightness->priv->levels - 1);
 
 	ret = dbus_g_proxy_call (proxy, "SetBrightness", &error,
-				 G_TYPE_UINT, brightness_level_hw,
+				 G_TYPE_INT, brightness_level_hw,
 				 G_TYPE_INVALID,
 				 G_TYPE_INVALID);
 	if (error) {
@@ -703,7 +703,7 @@ gpm_brightness_kbd_init (GpmBrightnessKbd *brightness)
 	}
 
 	/* save udi of kbd adapter */
-	gpm_hal_device_find_capability (brightness->priv->hal, "keyboard_backlight", &names);
+	gpm_hal_device_find_capability (brightness->priv->hal, "keyboard_backlight", &names, NULL);
 	if (names == NULL || names[0] == NULL) {
 		gpm_warning ("No devices of capability keyboard_backlight");
 		return;
@@ -727,7 +727,7 @@ gpm_brightness_kbd_init (GpmBrightnessKbd *brightness)
 
 	/* get levels that the adapter supports -- this does not change ever */
 	gpm_hal_device_get_uint (brightness->priv->hal, brightness->priv->udi, "keyboard_backlight.num_levels",
-				 &brightness->priv->levels);
+				 &brightness->priv->levels, NULL);
 
 	/* this changes under our feet */
 	gpm_brightness_kbd_get_hw (brightness, &brightness->priv->current_hw);
@@ -759,7 +759,7 @@ gpm_brightness_kbd_has_hw (void)
 
 	/* okay, as singleton - so we don't allocate more memory */
 	hal = gpm_hal_new ();
-	gpm_hal_device_find_capability (hal, "keyboard_backlight", &names);
+	gpm_hal_device_find_capability (hal, "keyboard_backlight", &names, NULL);
 
 	/* nothing found */
 	if (names == NULL || names[0] == NULL) {
