@@ -37,7 +37,7 @@
 #include <gdk/gdkx.h>
 
 #ifdef HAVE_DPMS_EXTENSION
-#include <X11/Xproto.h>			/* for CARD16 */
+#include <X11/Xproto.h>
 #include <X11/extensions/dpms.h>
 #include <X11/extensions/dpmsstr.h>
 #endif
