@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2006-2007 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -37,6 +37,7 @@ struct GpmGraphWidgetPrivate
 {
 	gboolean		 use_grid;
 	gboolean		 use_legend;
+	gboolean		 use_axis_labels;
 	gboolean		 use_events;
 
 	gboolean		 invert_x;
@@ -57,12 +58,16 @@ struct GpmGraphWidgetPrivate
 	gfloat			 unit_x; /* 10th width of graph */
 	gfloat			 unit_y; /* 10th width of graph */
 
-	GpmGraphWidgetAxisType	 axis_x;
-	GpmGraphWidgetAxisType	 axis_y;
+	GpmGraphWidgetAxisType	 axis_type_x;
+	GpmGraphWidgetAxisType	 axis_type_y;
+	const gchar		*axis_label_x;
+	const gchar		*axis_label_y;
+	gchar			*title;
+
 	cairo_t			*cr;
 	cairo_font_options_t	*options;
 
-	GList			*list;
+	GPtrArray		*data_list;
 	GList			*events;
 };
 
@@ -137,7 +142,7 @@ gpm_graph_widget_key_add (GpmGraphWidget       *graph,
 
 /**
  * gpm_graph_widget_string_to_axis_type:
- * @graph: This graph class instance
+ * @graph: This class instance
  * @type: The axis type, e.g. "percentage"
  *
  * Return value: The enumerated axis type
@@ -163,34 +168,98 @@ gpm_graph_widget_string_to_axis_type (const gchar *type)
 }
 
 /**
- * gpm_graph_widget_set_axis_x:
- * @graph: This graph class instance
- * @axis: The axis type, e.g. GPM_GRAPH_WIDGET_TYPE_TIME
+ * gpm_graph_widget_get_axis_label_y:
+ * @graph: This class instance
+ * @type: The axis type, e.g. GPM_GRAPH_WIDGET_TYPE_TIME
  **/
-void
-gpm_graph_widget_set_axis_x (GpmGraphWidget *graph, GpmGraphWidgetAxisType axis)
+static const gchar *
+gpm_graph_widget_get_axis_label_y (GpmGraphWidget *graph, GpmGraphWidgetAxisType type)
 {
-	g_return_if_fail (graph != NULL);
-	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
-	graph->priv->axis_x = axis;
+	g_return_val_if_fail (graph != NULL, NULL);
+	g_return_val_if_fail (GPM_IS_GRAPH_WIDGET (graph), NULL);
+
+	if (type == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
+		return _("Charge percentage");
+	}
+	if (type == GPM_GRAPH_WIDGET_TYPE_TIME) {
+		return _("Time remaining");
+	}
+	if (type == GPM_GRAPH_WIDGET_TYPE_POWER) {
+		return _("Power");
+	}
+	if (type == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
+		return _("Cell Voltage");
+	}
+	return _("Unknown caption");
 }
 
 /**
- * gpm_graph_widget_set_axis_y:
- * @graph: This graph class instance
+ * gpm_graph_widget_get_axis_label_x:
+ * @graph: This class instance
+ * @type: The axis type, e.g. GPM_GRAPH_WIDGET_TYPE_TIME
+ **/
+static const gchar *
+gpm_graph_widget_get_axis_label_x (GpmGraphWidget *graph, GpmGraphWidgetAxisType type)
+{
+	g_return_val_if_fail (graph != NULL, NULL);
+	g_return_val_if_fail (GPM_IS_GRAPH_WIDGET (graph), NULL);
+
+	if (type == GPM_GRAPH_WIDGET_TYPE_TIME) {
+		/* I want this translated please */
+		const char *moo;
+		moo = _("Time");
+		return _("Time since startup");
+	}
+	return _("Unknown caption");
+}
+
+/**
+ * gpm_graph_widget_set_title:
+ * @graph: This class instance
  * @axis: The axis type, e.g. GPM_GRAPH_WIDGET_TYPE_TIME
  **/
 void
-gpm_graph_widget_set_axis_y (GpmGraphWidget *graph, GpmGraphWidgetAxisType axis)
+gpm_graph_widget_set_title (GpmGraphWidget *graph, const gchar *title)
 {
 	g_return_if_fail (graph != NULL);
 	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
-	graph->priv->axis_y = axis;
+	if (graph->priv->title != NULL) {
+		g_free (graph->priv->title);
+	}
+	graph->priv->title = g_strdup (title);
+}
+
+/**
+ * gpm_graph_widget_set_axis_type_x:
+ * @graph: This class instance
+ * @axis: The axis type, e.g. GPM_GRAPH_WIDGET_TYPE_TIME
+ **/
+void
+gpm_graph_widget_set_axis_type_x (GpmGraphWidget *graph, GpmGraphWidgetAxisType axis)
+{
+	g_return_if_fail (graph != NULL);
+	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
+	graph->priv->axis_type_x = axis;
+	graph->priv->axis_label_x = gpm_graph_widget_get_axis_label_x (graph, axis);
+}
+
+/**
+ * gpm_graph_widget_set_axis_type_y:
+ * @graph: This class instance
+ * @axis: The axis type, e.g. GPM_GRAPH_WIDGET_TYPE_TIME
+ **/
+void
+gpm_graph_widget_set_axis_type_y (GpmGraphWidget *graph, GpmGraphWidgetAxisType axis)
+{
+	g_return_if_fail (graph != NULL);
+	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
+	graph->priv->axis_type_y = axis;
+	graph->priv->axis_label_y = gpm_graph_widget_get_axis_label_y (graph, axis);
 }
 
 /**
  * gpm_graph_widget_enable_legend:
- * @graph: This graph class instance
+ * @graph: This class instance
  * @enable: If we should show the legend
  **/
 void
@@ -204,9 +273,25 @@ gpm_graph_widget_enable_legend (GpmGraphWidget *graph, gboolean enable)
 	gtk_widget_show (GTK_WIDGET (graph));
 }
 
+/**
+ * gpm_graph_widget_enable_axis_labels:
+ * @graph: This class instance
+ * @enable: If we should show the axis labels
+ **/
+void
+gpm_graph_widget_enable_axis_labels (GpmGraphWidget *graph, gboolean enable)
+{
+	g_return_if_fail (graph != NULL);
+	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
+	graph->priv->use_axis_labels = enable;
+
+	gtk_widget_hide (GTK_WIDGET (graph));
+	gtk_widget_show (GTK_WIDGET (graph));
+}
+
 /**
  * gpm_graph_widget_enable_events:
- * @graph: This graph class instance
+ * @graph: This class instance
  * @enable: If we should show the legend
  **/
 void
@@ -238,7 +323,7 @@ gpm_graph_widget_class_init (GpmGraphWidgetClass *class)
 
 /**
  * gpm_graph_widget_init:
- * @graph: This graph class instance
+ * @graph: This class instance
  **/
 static void
 gpm_graph_widget_init (GpmGraphWidget *graph)
@@ -252,11 +337,15 @@ gpm_graph_widget_init (GpmGraphWidget *graph)
 	graph->priv->stop_y = 100;
 	graph->priv->use_grid = TRUE;
 	graph->priv->use_legend = FALSE;
+	graph->priv->use_axis_labels = FALSE;
 	graph->priv->autorange_x = TRUE;
-	graph->priv->list = NULL;
+	graph->priv->data_list = g_ptr_array_new ();
 	graph->priv->keyvals = NULL;
-	graph->priv->axis_x = GPM_GRAPH_WIDGET_TYPE_TIME;
-	graph->priv->axis_y = GPM_GRAPH_WIDGET_TYPE_PERCENTAGE;
+	graph->priv->axis_label_x = NULL;
+	graph->priv->axis_label_y = NULL;
+	graph->priv->title = NULL;
+	graph->priv->axis_type_x = GPM_GRAPH_WIDGET_TYPE_TIME;
+	graph->priv->axis_type_y = GPM_GRAPH_WIDGET_TYPE_PERCENTAGE;
 	/* setup font */
 	graph->priv->options = cairo_font_options_create ();
 }
@@ -279,6 +368,9 @@ gpm_graph_widget_finalize (GObject *object)
 		g_free (keyitem);
 	}
 	g_slist_free (graph->priv->keyvals);
+	g_ptr_array_free (graph->priv->data_list, FALSE);
+
+	g_free (graph->priv->title);
 
 	if (graph->priv->events) {
 		g_list_free (graph->priv->events);
@@ -287,7 +379,7 @@ gpm_graph_widget_finalize (GObject *object)
 
 /**
  * gpm_graph_widget_set_invert_x:
- * @graph: This graph class instance
+ * @graph: This class instance
  * @inv: If we should invert the axis
  *
  * Sets the inverse policy for the X axis, i.e. to count from 0..X or X..0
@@ -302,7 +394,7 @@ gpm_graph_widget_set_invert_x (GpmGraphWidget *graph, gboolean inv)
 
 /**
  * gpm_graph_widget_set_invert_y:
- * @graph: This graph class instance
+ * @graph: This class instance
  * @inv: If we should invert the axis
  *
  * Sets the inverse policy for the Y axis, i.e. to count from 0..Y or Y..0
@@ -317,25 +409,32 @@ gpm_graph_widget_set_invert_y (GpmGraphWidget *graph, gboolean inv)
 
 /**
  * gpm_graph_widget_set_data:
- * @graph: This graph class instance
+ * @graph: This class instance
  * @list: The GList values to be plotted on the graph
  *
  * Sets the data for the graph. You MUST NOT free the list before the widget.
  **/
 void
-gpm_graph_widget_set_data (GpmGraphWidget *graph, GList *list)
+gpm_graph_widget_set_data (GpmGraphWidget *graph, GList *list, guint id)
 {
 	g_return_if_fail (graph != NULL);
 	g_return_if_fail (GPM_IS_GRAPH_WIDGET (graph));
-	if (graph->priv->list) {
-		g_list_free (graph->priv->list);
+
+	/* fresh list */
+	if (graph->priv->data_list->len == 0 || graph->priv->data_list->len < id + 1) {
+		g_ptr_array_add (graph->priv->data_list, (gpointer) list);
+	} else {
+		/* remove existing, and add new */
+		gpm_debug ("Re-assigning dataset");
+		g_ptr_array_remove_index (graph->priv->data_list, id);
+		g_ptr_array_add (graph->priv->data_list, (gpointer) list);
 	}
-	graph->priv->list = list;
+
 }
 
 /**
  * gpm_graph_widget_set_events:
- * @graph: This graph class instance
+ * @graph: This class instance
  * @list: The GList events to be plotted on the graph
  *
  * Sets the data for the graph. You MUST NOT free the list before the widget.
@@ -403,14 +502,14 @@ gpm_get_axis_label (GpmGraphWidgetAxisType axis, gint value)
 		/*Translators: This is %i Volts*/
 		text = g_strdup_printf (_("%iV"), value / 1000);
 	} else {
-		text = g_strdup_printf ("%i??", value);
+		text = g_strdup_printf ("%i", value);
 	}
 	return text;
 }
 
 /**
  * gpm_graph_widget_draw_grid:
- * @graph: This graph class instance
+ * @graph: This class instance
  * @cr: Cairo drawing context
  *
  * Draw the 10x10 dotted grid onto the graph.
@@ -450,7 +549,7 @@ gpm_graph_widget_draw_grid (GpmGraphWidget *graph, cairo_t *cr)
 
 /**
  * gpm_graph_widget_draw_labels:
- * @graph: This graph class instance
+ * @graph: This class instance
  * @cr: Cairo drawing context
  *
  * Draw the X and the Y labels onto the graph.
@@ -482,7 +581,7 @@ gpm_graph_widget_draw_labels (GpmGraphWidget *graph, cairo_t *cr)
 		} else {
 			value = ((length_x / 10) * a) + graph->priv->start_x;
 		}
-		text = gpm_get_axis_label (graph->priv->axis_x, value);
+		text = gpm_get_axis_label (graph->priv->axis_type_x, value);
 
 		cairo_text_extents (cr, text, &extents);
 		/* have data points 0 and 10 bounded, but 1..9 centered */
@@ -505,11 +604,11 @@ gpm_graph_widget_draw_labels (GpmGraphWidget *graph, cairo_t *cr)
 	for (a=0; a<11; a++) {
 		b = graph->priv->box_y + (a * divheight);
 		if (graph->priv->invert_y) {
-			value = (length_y / 10) * a - graph->priv->start_y;
+			value = (length_y / 10) * a + graph->priv->start_y;
 		} else {
-			value = (length_y / 10) * (10 - a) - graph->priv->start_y;
+			value = (length_y / 10) * (10 - a) + graph->priv->start_y;
 		}
-		text = gpm_get_axis_label (graph->priv->axis_y, value);
+		text = gpm_get_axis_label (graph->priv->axis_type_y, value);
 
 		cairo_text_extents (cr, text, &extents);
 		/* have data points 0 and 10 bounded, but 1..9 centered */
@@ -529,46 +628,9 @@ gpm_graph_widget_draw_labels (GpmGraphWidget *graph, cairo_t *cr)
 	cairo_restore (cr);
 }
 
-/**
- * gpm_graph_widget_check_range:
- * @graph: This graph class instance
- *
- * Checks all points are displayable on the graph, nobbling if required.
- **/
-static void
-gpm_graph_widget_check_range (GpmGraphWidget *graph)
-{
-	GpmInfoDataPoint *new = NULL;
-	GList *l;
-
-	if (graph->priv->list == NULL || graph->priv->list->data == NULL) {
-		gpm_debug ("no data");
-		return;
-	}
-	for (l=graph->priv->list; l != NULL; l=l->next) {
-		new = (GpmInfoDataPoint *) l->data;
-		if (new->time < graph->priv->start_x) {
-			gpm_warning ("point out of range (x=%i)", new->time);
-			new->time = graph->priv->start_x;
-		}
-		if (new->time > graph->priv->stop_x) {
-			gpm_warning ("point out of range (x=%i)", new->time);
-			new->time = graph->priv->stop_x;
-		}
-		if (new->value < graph->priv->start_y) {
-			gpm_warning ("point out of range (y=%i)", new->value);
-			new->value = graph->priv->start_y;
-		}
-		if (new->value > graph->priv->stop_y) {
-			gpm_warning ("point out of range (y=%i)", new->value);
-			new->value = graph->priv->stop_y;
-		}
-	}
-}
-
 /**
  * gpm_graph_widget_auto_range:
- * @graph: This graph class instance
+ * @graph: This class instance
  *
  * Autoranges the graph axis depending on the axis type, and the maximum
  * value of the data. We have to be careful to choose a number that gives good
@@ -583,8 +645,10 @@ gpm_graph_widget_auto_range (GpmGraphWidget *graph)
 	gint smallest_y = 999999;
 	GpmInfoDataPoint *new = NULL;
 	GList *l;
+	GList *list;
+	guint i;
 
-	if (graph->priv->list == NULL || graph->priv->list->data == NULL) {
+	if (graph->priv->data_list->len == 0) {
 		gpm_debug ("no data");
 		graph->priv->start_x = 0;
 		graph->priv->start_y = 0;
@@ -593,21 +657,26 @@ gpm_graph_widget_auto_range (GpmGraphWidget *graph)
 		return;
 	}
 
-	for (l=graph->priv->list; l != NULL; l=l->next) {
-		new = (GpmInfoDataPoint *) l->data;
-		if (new->time > biggest_x) {
-			biggest_x = new->time;
-		}
-		if (new->value > biggest_y) {
-			biggest_y = new->value;
-		}
-		if (new->time < smallest_x) {
-			smallest_x = new->time;
-		}
-		if (new->value < smallest_y) {
-			smallest_y = new->value;
-		}
+	/* get the range for all graphs */
+	for (i=0; i<graph->priv->data_list->len; i++) {
+		list = (GList *) g_ptr_array_index (graph->priv->data_list, i);
+		for (l=list; l != NULL; l=l->next) {
+			new = (GpmInfoDataPoint *) l->data;
+			if (new->time > biggest_x) {
+				biggest_x = new->time;
+			}
+			if (new->value > biggest_y) {
+				biggest_y = new->value;
+			}
+			if (new->time < smallest_x) {
+				smallest_x = new->time;
+			}
+			if (new->value < smallest_y) {
+				smallest_y = new->value;
+			}
+		}		
 	}
+	gpm_debug ("Data range is %i<x<%i, %i<y<%i", smallest_x, biggest_x, smallest_y, biggest_y);
 
 	/* do we autorange the start (so it starts at non-zero)? */
 	if (graph->priv->autorange_x) {
@@ -622,29 +691,30 @@ gpm_graph_widget_auto_range (GpmGraphWidget *graph)
 	}
 
 	/* x */
-	if (graph->priv->axis_x == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
+	if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
 		graph->priv->stop_x = 100;
-	} else if (graph->priv->axis_x == GPM_GRAPH_WIDGET_TYPE_TIME) {
+	} else if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_TIME) {
 		graph->priv->stop_x = (((biggest_x - smallest_x) / (10 * 60)) + 1) * (10 * 60) + smallest_x;
-	} else if (graph->priv->axis_x == GPM_GRAPH_WIDGET_TYPE_POWER) {
+	} else if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_POWER) {
 		graph->priv->stop_x = (((biggest_x - smallest_x) / 10000) + 2) * 10000 + smallest_x;
 		if (graph->priv->stop_x < 10000) {
 			graph->priv->stop_x = 10000 + smallest_x;
 		}
-	} else if (graph->priv->axis_x == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
+	} else if (graph->priv->axis_type_x == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
 		graph->priv->stop_x = (((biggest_x - smallest_x) / 1000) + 2) * 1000 + smallest_x;
 		if (graph->priv->stop_x < 1000) {
 			graph->priv->stop_x = 1000 + smallest_x;
 		}
 	} else {
-		graph->priv->stop_x = ((biggest_x / 10) + 1) * 10 + smallest_x;
+		graph->priv->start_x = smallest_x;
+		graph->priv->stop_x = biggest_x;
 	}
 
 	/* y */
-	if (graph->priv->axis_y == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
+	if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_PERCENTAGE) {
 		graph->priv->start_y = 0;
 		graph->priv->stop_y = 100;
-	} else if (graph->priv->axis_y == GPM_GRAPH_WIDGET_TYPE_TIME) {
+	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_TIME) {
 		graph->priv->start_y = 0;
 		graph->priv->stop_y = ((biggest_y / 60) + 2)* 60;
 		if (graph->priv->stop_y > 60) {
@@ -653,25 +723,25 @@ gpm_graph_widget_auto_range (GpmGraphWidget *graph)
 		if (graph->priv->stop_y < 60) {
 			graph->priv->stop_y = 60;
 		}
-	} else if (graph->priv->axis_y == GPM_GRAPH_WIDGET_TYPE_POWER) {
+	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_POWER) {
 		graph->priv->start_y = 0;
 		graph->priv->stop_y = ((biggest_y / 10000) + 2) * 10000;
 		if (graph->priv->stop_y < 10000) {
 			graph->priv->stop_y = 10000;
 		}
-	} else if (graph->priv->axis_y == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
+	} else if (graph->priv->axis_type_y == GPM_GRAPH_WIDGET_TYPE_VOLTAGE) {
 		graph->priv->start_y = 0;
 		graph->priv->stop_y = ((biggest_y / 1000) + 2) * 1000;
 		if (graph->priv->stop_y < 1000) {
 			graph->priv->stop_y = 1000;
 		}
 	} else {
-		graph->priv->start_y = 0;
-		graph->priv->stop_y = ((biggest_y / 10) + 1) * 10;
-		if (graph->priv->stop_y < 10) {
-			graph->priv->stop_y = 10;
-		}
+		graph->priv->start_y = smallest_y;
+		graph->priv->stop_y = biggest_y;
 	}
+	gpm_debug ("Processed range is %i<x<%i, %i<y<%i",
+		   graph->priv->start_x, graph->priv->stop_x,
+		   graph->priv->start_y, graph->priv->stop_y);
 }
 
 /**
@@ -717,7 +787,7 @@ gpm_graph_widget_set_colour (cairo_t *cr, GpmGraphWidgetColour colour)
 	} else if (colour == GPM_GRAPH_WIDGET_COLOUR_DARK_GREY) {
 		cairo_set_source_rgb (cr, 0.5, 0.5, 0.5);
 	} else {
-		gpm_critical_error ("Unknown colour!");
+		gpm_warning ("Unknown colour: %i", colour);
 	}
 }
 
@@ -828,7 +898,7 @@ gpm_graph_widget_draw_legend_line (cairo_t *cr, gfloat x, gfloat y, GpmGraphWidg
 
 /**
  * gpm_graph_widget_get_pos_on_graph:
- * @graph: This graph class instance
+ * @graph: This class instance
  * @data_x: The data X-coordinate
  * @data_y: The data Y-coordinate
  * @x: The returned X position on the cairo surface
@@ -838,7 +908,7 @@ static void
 gpm_graph_widget_get_pos_on_graph (GpmGraphWidget *graph, gfloat data_x, gfloat data_y, float *x, float *y)
 {
 	*x = graph->priv->box_x + (graph->priv->unit_x * (data_x - graph->priv->start_x)) + 1;
-	*y = graph->priv->box_y + (graph->priv->unit_y * (gfloat)(graph->priv->stop_y - (data_y - graph->priv->start_y))) + 1.5;
+	*y = graph->priv->box_y + (graph->priv->unit_y * (gfloat)(graph->priv->stop_y - data_y)) + 1.5;
 }
 
 /**
@@ -892,7 +962,7 @@ gpm_graph_widget_interpolate_value (GpmInfoDataPoint *this,
 
 /**
  * gpm_graph_widget_draw_line:
- * @graph: This graph class instance
+ * @graph: This class instance
  * @cr: Cairo drawing context
  *
  * Draw the data line onto the graph with a big green line. We should already
@@ -907,35 +977,35 @@ gpm_graph_widget_draw_line (GpmGraphWidget *graph, cairo_t *cr)
 	GpmGraphWidgetKeyItem *keyitem;
 	GList *l;
 	GpmInfoDataPoint *new;
+	guint i;
 
-	if (graph->priv->list == NULL || graph->priv->list->data == NULL) {
+	if (graph->priv->data_list->len == 0) {
 		gpm_debug ("no data");
 		return;
 	}
-
-	/* I don't think we need to do this anymore.... */
-	if (FALSE) {
-		gpm_graph_widget_check_range (graph);
-	}
-
 	cairo_save (cr);
 
-	/* do the line on the graph */
-	l=graph->priv->list;
-	new = (GpmInfoDataPoint *) l->data;
-	gpm_graph_widget_get_pos_on_graph (graph, new->time, new->value, &oldx, &oldy);
-	for (l=l->next; l != NULL; l=l->next) {
+	/* do the lines on the graphs */
+	for (i=0; i<graph->priv->data_list->len; i++) {
+		gpm_debug ("drawing line %i", i);
+		l = (GList *) g_ptr_array_index (graph->priv->data_list, i);
 		new = (GpmInfoDataPoint *) l->data;
-		/* do line */
-		gpm_graph_widget_get_pos_on_graph (graph, new->time, new->value, &newx, &newy);
-		cairo_move_to (cr, oldx, oldy);
-		cairo_line_to (cr, newx, newy);
-		cairo_set_line_width (cr, 2);
-		gpm_graph_widget_set_colour (cr, new->colour);
-		cairo_stroke (cr);
-		/* save old */
-		oldx = newx;
-		oldy = newy;
+		oldx = 0;
+		oldy = 0;
+		gpm_graph_widget_get_pos_on_graph (graph, new->time, new->value, &oldx, &oldy);
+		for (l=l->next; l != NULL; l=l->next) {
+			new = (GpmInfoDataPoint *) l->data;
+			/* do line */
+			gpm_graph_widget_get_pos_on_graph (graph, new->time, new->value, &newx, &newy);
+			cairo_move_to (cr, oldx, oldy);
+			cairo_line_to (cr, newx, newy);
+			cairo_set_line_width (cr, 1.5);
+			gpm_graph_widget_set_colour (cr, new->colour);
+			cairo_stroke (cr);
+			/* save old */
+			oldx = newx;
+			oldy = newy;
+		}
 	}
 
 	/* only do the events sometimes */
@@ -946,7 +1016,8 @@ gpm_graph_widget_draw_line (GpmGraphWidget *graph, cairo_t *cr)
 		GpmInfoDataPoint *point_last = NULL;
 
 		/* we track the list so we can put the point on the line */
-		GList *l2 = graph->priv->list;
+		l = (GList *) g_ptr_array_index (graph->priv->data_list, 0);
+		GList *l2 = l;
 		if (l2) {
 			point_this = (GpmInfoDataPoint *) l2->data;
 			l2 = l2->next;
@@ -1094,7 +1165,7 @@ gpm_graph_widget_draw_legend (GpmGraphWidget *graph,
 
 /**
  * gpm_graph_widget_legend_calculate_width:
- * @graph: This graph class instance
+ * @graph: This class instance
  * @cr: Cairo drawing context
  * Return value: The width of the legend, including borders.
  *
@@ -1157,7 +1228,7 @@ gpm_graph_widget_legend_calculate_size (GpmGraphWidget *graph, cairo_t *cr,
 
 /**
  * gpm_graph_widget_draw_graph:
- * @graph: This graph class instance
+ * @graph: This class instance
  * @cr: Cairo drawing context
  *
  * Draw the complete graph, with the box, the grid, the labels and the line.
@@ -1171,6 +1242,9 @@ gpm_graph_widget_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 	guint legend_width = 0;
 	gint data_x;
 	gint data_y;
+	cairo_text_extents_t extents_axis_label_x;
+	cairo_text_extents_t extents_axis_label_y;
+	cairo_text_extents_t extents_title;
 
 	GpmGraphWidget *graph = (GpmGraphWidget*) graph_widget;
 	g_return_if_fail (graph != NULL);
@@ -1183,6 +1257,27 @@ gpm_graph_widget_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 	graph->priv->box_x = 35;
 	graph->priv->box_y = 5;
 
+	if (graph->priv->title != NULL) {
+		cairo_text_extents (cr, graph->priv->title, &extents_title);
+		graph->priv->box_y += extents_title.height;
+	}
+
+	graph->priv->box_height = graph_widget->allocation.height - (20 + graph->priv->box_y);
+
+	/* make size adjustment for axis labels */
+	if (graph->priv->use_axis_labels) {
+		/* get the size of the x axis text */
+		cairo_save (cr);
+		cairo_set_font_size (cr, 12);
+		cairo_text_extents (cr, graph->priv->axis_label_x, &extents_axis_label_x);
+		cairo_text_extents (cr, graph->priv->axis_label_y, &extents_axis_label_y);
+		cairo_restore (cr);
+
+		graph->priv->box_height -= (extents_axis_label_x.height + 2);
+		graph->priv->box_x += (extents_axis_label_y.height + 2);
+	}
+
+	/* make size adjustment for legend */
 	if (graph->priv->use_legend) {
 		graph->priv->box_width = graph_widget->allocation.width -
 					 (3 + legend_width + 5 + graph->priv->box_x);
@@ -1192,7 +1287,12 @@ gpm_graph_widget_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 		graph->priv->box_width = graph_widget->allocation.width -
 					 (3 + graph->priv->box_x);
 	}
-	graph->priv->box_height = graph_widget->allocation.height - (20 + graph->priv->box_y);
+
+	/* center title */
+	if (graph->priv->title != NULL) {
+		cairo_move_to (cr, graph->priv->box_x + (graph->priv->box_width-extents_title.width)/2, 9);
+		cairo_show_text (cr, graph->priv->title);
+	}
 
 	/* graph background */
 	gpm_graph_widget_draw_bounding_box (cr, graph->priv->box_x, graph->priv->box_y,
@@ -1211,6 +1311,20 @@ gpm_graph_widget_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 	graph->priv->unit_y = (float)(graph->priv->box_height - 3) / (float) data_y;
 
 	gpm_graph_widget_draw_labels (graph, cr);
+
+	if (graph->priv->use_axis_labels) {
+		/* horizontal axis label */
+		cairo_save (cr);
+		cairo_set_font_size (cr, 12);
+		cairo_move_to (cr, graph->priv->box_x + ((graph->priv->box_width - extents_axis_label_x.width)/2), graph_widget->allocation.height - 3);
+		cairo_show_text (cr, graph->priv->axis_label_x);
+		/* vertical axis label */
+		cairo_move_to (cr, extents_axis_label_y.height, ((graph->priv->box_height + extents_axis_label_y.width) / 2) - graph->priv->box_y);
+		cairo_rotate (cr, -3.1415927 / 2.0);
+		cairo_show_text (cr, graph->priv->axis_label_y);
+		cairo_restore (cr);
+	}
+
 	gpm_graph_widget_draw_line (graph, cr);
 
 	if (graph->priv->use_legend) {
@@ -1222,7 +1336,7 @@ gpm_graph_widget_draw_graph (GtkWidget *graph_widget, cairo_t *cr)
 
 /**
  * gpm_graph_widget_expose:
- * @graph: This graph class instance
+ * @graph: This class instance
  * @event: The expose event
  *
  * Just repaint the entire graph widget on expose.
