@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
- * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2005-2007 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -39,12 +39,11 @@
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 #include <libgnomeui/gnome-client.h>
-#include <libgnomeui/gnome-client.h>
-#include <gnome-keyring-1/gnome-keyring.h>
 #include "gpm-ac-adapter.h"
 #include "gpm-battery.h"
 #include "gpm-button.h"
 #include "gpm-conf.h"
+#include "gpm-control.h"
 #include "gpm-common.h"
 #include "gpm-debug.h"
 #include "gpm-dpms.h"
@@ -52,27 +51,26 @@
 #include "gpm-idle.h"
 #include "gpm-info.h"
 #include "gpm-inhibit.h"
-#include "gpm-networkmanager.h"
 #include "gpm-manager.h"
 #include "gpm-notify.h"
 #include "gpm-power.h"
-#include "gpm-policy.h"
 #include "gpm-powermanager.h"
 #include "gpm-prefs.h"
 #include "gpm-screensaver.h"
-#include "gpm-srv-brightness-lcd.h"
+#include "gpm-srv-backlight.h"
 #include "gpm-srv-brightness-kbd.h"
 #include "gpm-srv-cpufreq.h"
-#include "gpm-srv-dpms.h"
 #include "gpm-srv-screensaver.h"
 #include "gpm-stock-icons.h"
 #include "gpm-sound.h"
 #include "gpm-tray-icon.h"
 #include "gpm-warning.h"
 
-#include "dbus/gpm-dbus-dpms.h"
+#include "dbus/gpm-dbus-control.h"
 #include "dbus/gpm-dbus-statistics.h"
-#include "dbus/gpm-dbus-brightness-lcd.h"
+#include "dbus/gpm-dbus-backlight.h"
+#include "dbus/gpm-dbus-ui.h"
+#include "dbus/gpm-dbus-inhibit.h"
 
 static void     gpm_manager_class_init	(GpmManagerClass *klass);
 static void     gpm_manager_init	(GpmManager      *manager);
@@ -95,17 +93,16 @@ struct GpmManagerPrivate
 	GpmInhibit		*inhibit;
 	GpmNotify		*notify;
 	GpmPower		*power;
-	GpmPolicy		*policy;
+	GpmControl		*control;
 	GpmScreensaver 		*screensaver;
 	GpmSound 		*sound;
 	GpmTrayIcon		*tray_icon;
 	GpmWarning		*warning;
 
 	/* interactive services */
-	GpmSrvBrightnessLcd	*srv_brightness_lcd;
+	GpmSrvBacklight		*srv_backlight;
 	GpmSrvBrightnessKbd	*srv_brightness_kbd;
 	GpmSrvCpuFreq	 	*srv_cpufreq;
-	GpmSrvDpms	 	*srv_dpms;
 	GpmSrvScreensaver 	*srv_screensaver;
 
 	GpmWarningState		 last_primary;
@@ -115,9 +112,6 @@ struct GpmManagerPrivate
 	GpmWarningState		 last_pda;
 
 	gboolean		 done_notify_fully_charged;
-
-	time_t			 last_resume_event;
-	guint			 suppress_policy_timeout;
 };
 
 enum {
@@ -129,11 +123,6 @@ static guint	     signals [LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (GpmManager, gpm_manager, G_TYPE_OBJECT)
 
-/* prototypes */
-static gboolean gpm_manager_suspend (GpmManager *manager, GError **error);
-static gboolean gpm_manager_hibernate (GpmManager *manager, GError **error);
-static gboolean gpm_manager_shutdown (GpmManager *manager, GError **error);
-
 /**
  * gpm_manager_error_quark:
  * Return value: Our personal error quark.
@@ -148,45 +137,6 @@ gpm_manager_error_quark (void)
 	return quark;
 }
 
-/**
- * gpm_manager_is_policy_timout_valid:
- * @manager: This class instance
- * @action: The action we want to do, e.g. "suspend"
- *
- * Checks if the difference in time between this request for an action, and
- * the last action completing is larger than the timeout set in gconf.
- *
- * Return value: TRUE if we can perform the action.
- **/
-static gboolean
-gpm_manager_is_policy_timout_valid (GpmManager  *manager,
-				    const gchar *action)
-{
-	gchar *message;
-	if ((time (NULL) - manager->priv->last_resume_event) <=
-	    manager->priv->suppress_policy_timeout) {
-		message = g_strdup_printf ("Skipping suppressed %s", action);
-		gpm_info_event_log (manager->priv->info,
-				    GPM_EVENT_NOTIFICATION, message);
-		g_free (message);
-		return FALSE;
-	}
-	return TRUE;
-}
-
-/**
- * gpm_manager_reset_event_time:
- * @manager: This class instance
- *
- * Resets the time so we do not do any more actions until the
- * timeout has passed.
- **/
-static void
-gpm_manager_reset_event_time (GpmManager *manager)
-{
-	manager->priv->last_resume_event = time (NULL);
-}
-
 /**
  * gpm_manager_is_inhibit_valid:
  * @manager: This class instance
@@ -200,13 +150,16 @@ gpm_manager_reset_event_time (GpmManager *manager)
  **/
 static gboolean
 gpm_manager_is_inhibit_valid (GpmManager *manager,
+			      gboolean	  user_action,
 			      const char *action)
 {
-	gboolean action_ok;
+	gboolean valid;
 	gchar *title;
 
-	action_ok = gpm_inhibit_check (manager->priv->inhibit);
-	if (! action_ok) {
+	/* We have to decide on whether this is a idle action or a user keypress */
+	gpm_inhibit_is_valid (manager->priv->inhibit, user_action, &valid, NULL);
+
+	if (valid == FALSE) {
 		GString *message = g_string_new ("");
 		const char *msg;
 
@@ -223,76 +176,59 @@ gpm_manager_is_inhibit_valid (GpmManager *manager,
 		g_string_free (message, TRUE);
 		g_free (title);
 	}
-	return action_ok;
+	return valid;
 }
 
 /**
  * gpm_manager_allowed_suspend:
- * @manager: This class instance
- * @can: If we can suspend
  *
- * Checks the HAL key power_management.can_suspend_to_ram and also
- * checks gconf to see if we are allowed to suspend this computer.
+ * Proxy this to maintain API compatability.
  **/
 gboolean
 gpm_manager_allowed_suspend (GpmManager *manager,
 			     gboolean   *can,
 			     GError    **error)
 {
-	g_return_val_if_fail (can, FALSE);
-	gpm_policy_allowed_suspend (manager->priv->policy, can);
-	return TRUE;
+	return gpm_control_allowed_suspend (manager->priv->control, can, error);
 }
 
 /**
  * gpm_manager_allowed_hibernate:
- * @manager: This class instance
- * @can: If we can hibernate
  *
- * Checks the HAL key power_management.can_suspend_to_disk and also
- * checks gconf to see if we are allowed to hibernate this computer.
+ * Proxy this to maintain API compatability.
  **/
 gboolean
 gpm_manager_allowed_hibernate (GpmManager *manager,
 			       gboolean   *can,
 			       GError    **error)
 {
-	g_return_val_if_fail (can, FALSE);
-	gpm_policy_allowed_hibernate (manager->priv->policy, can);
-	return TRUE;
+	return gpm_control_allowed_hibernate (manager->priv->control, can, error);
 }
 
 /**
  * gpm_manager_allowed_shutdown:
- * @manager: This class instance
- * @can: If we can shutdown
  *
+ * Proxy this to maintain API compatability.
  **/
 gboolean
 gpm_manager_allowed_shutdown (GpmManager *manager,
 			      gboolean   *can,
 			      GError    **error)
 {
-	g_return_val_if_fail (can, FALSE);
-	gpm_policy_allowed_shutdown (manager->priv->policy, can);
-	return TRUE;
+	return gpm_control_allowed_shutdown (manager->priv->control, can, error);
 }
 
 /**
  * gpm_manager_allowed_reboot:
- * @manager: This class instance
- * @can: If we can reboot
  *
- * Stub function -- TODO.
+ * Proxy this to maintain API compatability.
  **/
 gboolean
 gpm_manager_allowed_reboot (GpmManager *manager,
 			    gboolean   *can,
 			    GError    **error)
 {
-	g_return_val_if_fail (can, FALSE);
-	gpm_policy_allowed_reboot (manager->priv->policy, can);
-	return TRUE;
+	return gpm_control_allowed_reboot (manager->priv->control, can, error);
 }
 
 /**
@@ -331,37 +267,6 @@ gpm_manager_sync_policy_sleep (GpmManager *manager)
 	gpm_idle_set_system_timeout (manager->priv->idle, sleep_computer);
 }
 
-/**
- * gpm_manager_get_lock_policy:
- * @manager: This class instance
- * @policy: The policy gconf string.
- *
- * This function finds out if we should lock the screen when we do an
- * action. It is required as we can either use the gnome-screensaver policy
- * or the custom policy. See the yelp file for more information.
- *
- * Return value: TRUE if we should lock.
- **/
-static gboolean
-gpm_manager_get_lock_policy (GpmManager  *manager,
-			     const gchar *policy)
-{
-	gboolean do_lock;
-	gboolean use_ss_setting;
-	/* This allows us to over-ride the custom lock settings set in gconf
-	   with a system default set in gnome-screensaver.
-	   See bug #331164 for all the juicy details. :-) */
-	gpm_conf_get_bool (manager->priv->conf, GPM_CONF_LOCK_USE_SCREENSAVER, &use_ss_setting);
-	if (use_ss_setting) {
-		do_lock = gpm_screensaver_lock_enabled (manager->priv->screensaver);
-		gpm_debug ("Using ScreenSaver settings (%i)", do_lock);
-	} else {
-		gpm_conf_get_bool (manager->priv->conf, policy, &do_lock);
-		gpm_debug ("Using custom locking settings (%i)", do_lock);
-	}
-	return do_lock;
-}
-
 /**
  * gpm_manager_blank_screen:
  * @manager: This class instance
@@ -381,9 +286,9 @@ gpm_manager_blank_screen (GpmManager *manager,
 	gboolean ret = TRUE;
 	GError  *error = NULL;
 
-	do_lock = gpm_manager_get_lock_policy (manager,
+	do_lock = gpm_control_get_lock_policy (manager->priv->control,
 					       GPM_CONF_LOCK_ON_BLANK_SCREEN);
-	if (do_lock) {
+	if (do_lock == TRUE) {
 		if (!gpm_screensaver_lock (manager->priv->screensaver))
 			gpm_debug ("Could not lock screen via gnome-screensaver");
 	}
@@ -420,39 +325,13 @@ gpm_manager_unblank_screen (GpmManager *manager,
 		ret = FALSE;
 	}
 
-	do_lock = gpm_manager_get_lock_policy (manager, GPM_CONF_LOCK_ON_BLANK_SCREEN);
-	if (do_lock) {
+	do_lock = gpm_control_get_lock_policy (manager->priv->control, GPM_CONF_LOCK_ON_BLANK_SCREEN);
+	if (do_lock == TRUE) {
 		gpm_screensaver_poke (manager->priv->screensaver);
 	}
 	return ret;
 }
 
-/**
- * manager_explain_reason:
- * @manager: This class instance
- * @event: The event type, e.g. GPM_EVENT_DPMS_OFF
- * @pre: The action we are about to do, e.g. "Suspending computer"
- * @post: The reason we are performing the policy action, e.g. "battery critical"
- *
- * Helper function
- **/
-static void
-manager_explain_reason (GpmManager   *manager,
-			GpmGraphWidgetEvent event,
-			const gchar  *pre,
-			const gchar  *post)
-{
-	gchar *message;
-	if (post) {
-		message = g_strdup_printf (_("%s because %s"), pre, post);
-	} else {
-		message = g_strdup (pre);
-	}
-	gpm_syslog (message);
-	gpm_info_event_log (manager->priv->info, event, message);
-	g_free (message);
-}
-
 /**
  * manager_policy_do:
  * @manager: This class instance
@@ -474,34 +353,34 @@ manager_policy_do (GpmManager  *manager,
 	if (action == NULL) {
 		return;
 	}
-	if (gpm_manager_is_policy_timout_valid (manager, "policy event") == FALSE) {
+	if (gpm_control_is_policy_timout_valid (manager->priv->control, "policy event") == FALSE) {
 		return;
 	}
 
 	if (strcmp (action, ACTION_NOTHING) == 0) {
-		manager_explain_reason (manager, GPM_EVENT_NOTIFICATION,
+		gpm_info_explain_reason (manager->priv->info, GPM_EVENT_NOTIFICATION,
 					_("Doing nothing"), reason);
 
 	} else if (strcmp (action, ACTION_SUSPEND) == 0) {
-		manager_explain_reason (manager, GPM_EVENT_SUSPEND,
+		gpm_info_explain_reason (manager->priv->info, GPM_EVENT_SUSPEND,
 					_("Suspending computer"), reason);
-		gpm_manager_suspend (manager, NULL);
+		gpm_control_suspend (manager->priv->control, NULL);
 
 	} else if (strcmp (action, ACTION_HIBERNATE) == 0) {
-		manager_explain_reason (manager, GPM_EVENT_HIBERNATE,
+		gpm_info_explain_reason (manager->priv->info, GPM_EVENT_HIBERNATE,
 					_("Hibernating computer"), reason);
-		gpm_manager_hibernate (manager, NULL);
+		gpm_control_hibernate (manager->priv->control, NULL);
 
 	} else if (strcmp (action, ACTION_BLANK) == 0) {
 		gpm_manager_blank_screen (manager, NULL);
 
 	} else if (strcmp (action, ACTION_SHUTDOWN) == 0) {
-		manager_explain_reason (manager, GPM_EVENT_NOTIFICATION,
+		gpm_info_explain_reason (manager->priv->info, GPM_EVENT_NOTIFICATION,
 					_("Shutting down computer"), reason);
-		gpm_manager_shutdown (manager, NULL);
+		gpm_control_shutdown (manager->priv->control, NULL);
 
 	} else if (strcmp (action, ACTION_INTERACTIVE) == 0) {
-		manager_explain_reason (manager, GPM_EVENT_NOTIFICATION,
+		gpm_info_explain_reason (manager->priv->info, GPM_EVENT_NOTIFICATION,
 					_("GNOME interactive logout"), reason);
 		gnome_client_request_save (gnome_master_client (),
 					   GNOME_SAVE_GLOBAL,
@@ -573,13 +452,8 @@ gpm_manager_get_low_power_mode (GpmManager  *manager,
 
 /**
  * gpm_manager_inhibit:
- * @manager: This class instance
- * @application: The application that sent the request, e.g. "Nautilus"
- * @reason: The reason given to inhibit, e.g. "Copying files"
- * @context: The context we are talking to
  *
- * Processes an inhibit request from an application that want to stop the
- * idle action suspend from happening.
+ * Proxy this while we support the old API.
  **/
 void
 gpm_manager_inhibit (GpmManager	 *manager,
@@ -588,20 +462,13 @@ gpm_manager_inhibit (GpmManager	 *manager,
 		     DBusGMethodInvocation *context,
 		     GError     **error)
 {
-	guint32 cookie;
-	const gchar *connection = dbus_g_method_get_sender (context);
-	cookie = gpm_inhibit_add (manager->priv->inhibit, connection, application, reason);
-	dbus_g_method_return (context, cookie);
+	gpm_inhibit_inhibit_auto (manager->priv->inhibit, application, reason, context, error);
 }
 
 /**
  * gpm_manager_uninhibit:
- * @manager: This class instance
- * @cookie: The application cookie, e.g. 17534
- * @context: The context we are talking to
  *
- * Processes an allow request from an application that want to allow the
- * idle action suspend to happen.
+ * Proxy this while we support the old API.
  **/
 void
 gpm_manager_uninhibit (GpmManager	 *manager,
@@ -609,347 +476,72 @@ gpm_manager_uninhibit (GpmManager	 *manager,
 		       DBusGMethodInvocation *context,
 		       GError		**error)
 {
-	const gchar *connection = dbus_g_method_get_sender (context);
-	gpm_inhibit_remove (manager->priv->inhibit, connection, cookie);
+	gpm_inhibit_un_inhibit (manager->priv->inhibit, cookie, error);
 	dbus_g_method_return (context);
 }
 
-/**
- * gpm_manager_shutdown:
- * @manager: This class instance
- *
- * Shuts down the computer, saving the session if possible.
- **/
-static gboolean
-gpm_manager_shutdown (GpmManager *manager,
-		      GError    **error)
-{
-	gboolean allowed;
-	gboolean ret;
-	gboolean save_session;
-
-	gpm_policy_allowed_shutdown (manager->priv->policy, &allowed);
-	if (! allowed) {
-		gpm_warning ("Cannot shutdown");
-		g_set_error (error,
-			     GPM_MANAGER_ERROR,
-			     GPM_MANAGER_ERROR_GENERAL,
-			     "Cannot shutdown");
-		return FALSE;
-	}
-
-	gpm_conf_get_bool (manager->priv->conf, GPM_CONF_SESSION_REQUEST_SAVE, &save_session);
-	/* We can set g-p-m to not save the session to avoid confusing new
-	   users. By default we save the session to preserve data. */
-	if (save_session == TRUE) {
-		gnome_client_request_save (gnome_master_client (),
-					   GNOME_SAVE_GLOBAL,
-					   FALSE, GNOME_INTERACT_NONE, FALSE,  TRUE);
-	}
-	gpm_hal_shutdown (manager->priv->hal);
-	ret = TRUE;
-
-	return ret;
-}
-
-/**
- * gpm_manager_reboot:
- * @manager: This class instance
- *
- * Reboots the computer, saving the session if possible.
- **/
-static gboolean
-gpm_manager_reboot (GpmManager *manager,
-		    GError    **error)
-{
-	gboolean allowed;
-	gboolean ret;
-	gboolean save_session;
-
-	gpm_policy_allowed_reboot (manager->priv->policy, &allowed);
-	if (! allowed) {
-		gpm_warning ("Cannot reboot");
-		g_set_error (error,
-			     GPM_MANAGER_ERROR,
-			     GPM_MANAGER_ERROR_GENERAL,
-			     "Cannot reboot");
-		return FALSE;
-	}
-
-	gpm_conf_get_bool (manager->priv->conf, GPM_CONF_SESSION_REQUEST_SAVE, &save_session);
-	/* We can set g-p-m to not save the session to avoid confusing new
-	   users. By default we save the session to preserve data. */
-	if (save_session == TRUE) {
-		gnome_client_request_save (gnome_master_client (),
-					   GNOME_SAVE_GLOBAL,
-					   FALSE, GNOME_INTERACT_NONE, FALSE,  TRUE);
-	}
-
-	gpm_hal_reboot (manager->priv->hal);
-	ret = TRUE;
-
-	return ret;
-}
-
-/**
- * gpm_manager_hibernate:
- * @manager: This class instance
- *
- * We want to hibernate the computer. This function deals with the "fluff" -
- * like disconnecting the networks and locking the screen, then does the
- * hibernate using HAL, then handles all the resume actions.
- *
- * Return value: If the hibernate was successful.
- **/
-static gboolean
-gpm_manager_hibernate (GpmManager *manager,
-		       GError    **error)
-{
-	gboolean allowed;
-	gboolean ret;
-	gboolean do_lock;
-        gboolean nm_sleep;
-	GnomeKeyringResult keyres;
-
-	gpm_policy_allowed_hibernate (manager->priv->policy, &allowed);
-
-	if (! allowed) {
-		gpm_syslog ("cannot hibernate as not allowed from policy");
-		g_set_error (error,
-			     GPM_MANAGER_ERROR,
-			     GPM_MANAGER_ERROR_GENERAL,
-			     "Cannot hibernate");
-		gpm_sound_event (manager->priv->sound, GPM_SOUND_SUSPEND_FAILURE);
-		return FALSE;
-	}
-
-	/* we should lock keyrings when sleeping #375681 */
-	keyres = gnome_keyring_lock_all_sync ();
-	if (keyres != GNOME_KEYRING_RESULT_OK) {
-		gpm_debug ("could not lock keyring");
-	}
-
-	do_lock = gpm_manager_get_lock_policy (manager,
-					       GPM_CONF_LOCK_ON_HIBERNATE);
-	if (do_lock) {
-		gpm_screensaver_lock (manager->priv->screensaver);
-	}
-
-	gpm_conf_get_bool (manager->priv->conf,  GPM_CONF_NETWORKMANAGER_SLEEP, &nm_sleep);
-	if (nm_sleep) {
-		gpm_networkmanager_sleep ();
-	}
-
-	ret = gpm_hal_hibernate (manager->priv->hal);
-	manager_explain_reason (manager, GPM_EVENT_RESUME,
-				_("Resuming computer"), NULL);
-
-	/* we need to refresh all the power caches */
-	gpm_power_update_all (manager->priv->power);
-
-	if (! ret) {
-		gchar *message;
-		gboolean show_notify;
-
-		gpm_sound_event (manager->priv->sound, GPM_SOUND_SUSPEND_FAILURE);
-
-		/* We only show the HAL failed notification if set in gconf */
-		gpm_conf_get_bool (manager->priv->conf, GPM_CONF_NOTIFY_HAL_ERROR, &show_notify);
-		if (show_notify) {
-			const gchar *title = _("Hibernate Problem");
-			message = g_strdup_printf (_("HAL failed to %s. "
-						     "Check the help file for common problems."),
-						     _("hibernate"));
-			gpm_syslog ("hibernate failed");
-			gpm_notify_display (manager->priv->notify,
-					      title,
-					      message,
-					      GPM_NOTIFY_TIMEOUT_LONG,
-					      GTK_STOCK_DIALOG_WARNING,
-					      GPM_NOTIFY_URGENCY_LOW);
-			gpm_info_event_log (manager->priv->info,
-					    GPM_EVENT_NOTIFICATION, title);
-			g_free (message);
-		}
-	}
-
-	if (do_lock) {
-		gpm_screensaver_poke (manager->priv->screensaver);
-	}
-
-	gpm_conf_get_bool (manager->priv->conf, GPM_CONF_NETWORKMANAGER_SLEEP, &nm_sleep);
-	if (nm_sleep) {
-		gpm_networkmanager_wake ();
-	}
-
-	gpm_srv_dpms_sync_policy (manager->priv->srv_dpms);
-
-	/* save the time that we resumed */
-	gpm_manager_reset_event_time (manager);
-
-	return ret;
-}
-
-/**
- * gpm_manager_suspend:
- * @manager: This class instance
- *
- * We want to suspend the computer. This function deals with the "fluff" -
- * like disconnecting the networks and locking the screen, then does the
- * suspend using HAL, then handles all the resume actions.
- *
- * Return value: If the suspend was successful.
- **/
-static gboolean
-gpm_manager_suspend (GpmManager *manager,
-		     GError    **error)
-{
-	gboolean allowed;
-	gboolean ret;
-	gboolean do_lock;
-	gboolean nm_sleep;
-	GnomeKeyringResult keyres;
-
-	gpm_policy_allowed_suspend (manager->priv->policy, &allowed);
-
-	if (! allowed) {
-		gpm_syslog ("cannot suspend as not allowed from policy");
-		g_set_error (error,
-			     GPM_MANAGER_ERROR,
-			     GPM_MANAGER_ERROR_GENERAL,
-			     "Cannot suspend");
-		gpm_sound_event (manager->priv->sound, GPM_SOUND_SUSPEND_FAILURE);
-		return FALSE;
-	}
-
-	/* we should lock keyrings when sleeping #375681 */
-	keyres = gnome_keyring_lock_all_sync ();
-	if (keyres != GNOME_KEYRING_RESULT_OK) {
-		gpm_debug ("could not lock keyring");
-	}
-
-	do_lock = gpm_manager_get_lock_policy (manager,
-					       GPM_CONF_LOCK_ON_SUSPEND);
-	if (do_lock) {
-		gpm_screensaver_lock (manager->priv->screensaver);
-	}
-
-	gpm_conf_get_bool (manager->priv->conf, GPM_CONF_NETWORKMANAGER_SLEEP, &nm_sleep);
-	if (nm_sleep) {
-		gpm_networkmanager_sleep ();
-	}
-
-	/* Do the suspend */
-	ret = gpm_hal_suspend (manager->priv->hal, 0);
-	manager_explain_reason (manager, GPM_EVENT_RESUME,
-				_("Resuming computer"), NULL);
-
-	/* We need to refresh all the power caches */
-	gpm_power_update_all (manager->priv->power);
-
-	if (! ret) {
-		gboolean show_notify;
-
-		gpm_sound_event (manager->priv->sound, GPM_SOUND_SUSPEND_FAILURE);
-
-		/* We only show the HAL failed notification if set in gconf */
-		gpm_conf_get_bool (manager->priv->conf, GPM_CONF_NOTIFY_HAL_ERROR, &show_notify);
-		if (show_notify) {
-			const gchar *title;
-			gchar *message;
-			message = g_strdup_printf (_("Your computer failed to %s.\n"
-						     "Check the help file for common problems."),
-						     _("suspend"));
-			title = _("Suspend Problem");
-			gpm_syslog ("suspend failed");
-			gpm_notify_display (manager->priv->notify,
-					      title,
-					      message,
-					      GPM_NOTIFY_TIMEOUT_LONG,
-					      GTK_STOCK_DIALOG_WARNING,
-					      GPM_NOTIFY_URGENCY_LOW);
-			gpm_info_event_log (manager->priv->info,
-					    GPM_EVENT_NOTIFICATION,
-					    title);
-			g_free (message);
-			gpm_sound_event (manager->priv->sound, GPM_SOUND_SUSPEND_FAILURE);
-		}
-	}
-
-	if (do_lock) {
-		gpm_screensaver_poke (manager->priv->screensaver);
-	}
-
-	gpm_conf_get_bool (manager->priv->conf, GPM_CONF_NETWORKMANAGER_SLEEP, &nm_sleep);
-	if (nm_sleep) {
-		gpm_networkmanager_wake ();
-	}
-
-	gpm_srv_dpms_sync_policy (manager->priv->srv_dpms);
-
-	/* save the time that we resumed */
-	gpm_manager_reset_event_time (manager);
-
-	return ret;
-}
-
 /**
  * gpm_manager_suspend_dbus_method:
- * @manager: This class instance
+ *
+ * Proxy this while we support the old API.
  **/
 gboolean
 gpm_manager_suspend_dbus_method (GpmManager *manager,
 				 GError    **error)
 {
 	/* FIXME: From where? */
-	manager_explain_reason (manager, GPM_EVENT_SUSPEND,
+	gpm_info_explain_reason (manager->priv->info, GPM_EVENT_SUSPEND,
 				_("Suspending computer"),
 				_("the DBUS method Suspend() was invoked"));
-	return gpm_manager_suspend (manager, error);
+	return gpm_control_suspend (manager->priv->control, error);
 }
 
 /**
  * gpm_manager_hibernate_dbus_method:
+ *
+ * Proxy this while we support the old API.
  **/
 gboolean
 gpm_manager_hibernate_dbus_method (GpmManager *manager,
 				   GError    **error)
 {
 	/* FIXME: From where? */
-	manager_explain_reason (manager, GPM_EVENT_HIBERNATE,
+	gpm_info_explain_reason (manager->priv->info, GPM_EVENT_HIBERNATE,
 				_("Hibernating computer"),
 				_("the DBUS method Hibernate() was invoked"));
-	return gpm_manager_hibernate (manager, error);
+	return gpm_control_hibernate (manager->priv->control, error);
 }
 
 /**
  * gpm_manager_shutdown_dbus_method:
- * @manager: This class instance
+ *
+ * Proxy this while we support the old API.
  **/
 gboolean
 gpm_manager_shutdown_dbus_method (GpmManager *manager,
 				  GError    **error)
 {
 	/* FIXME: From where? */
-	manager_explain_reason (manager, GPM_EVENT_NOTIFICATION,
+	gpm_info_explain_reason (manager->priv->info, GPM_EVENT_NOTIFICATION,
 				_("Shutting down computer"),
 				_("the DBUS method Shutdown() was invoked"));
-	return gpm_manager_shutdown (manager, error);
+	return gpm_control_shutdown (manager->priv->control, error);
 }
 
 /**
  * gpm_manager_reboot_dbus_method:
- * @manager: This class instance
+ *
+ * Proxy this while we support the old API.
  **/
 gboolean
 gpm_manager_reboot_dbus_method (GpmManager *manager,
 				GError    **error)
 {
 	/* FIXME: From where? */
-	manager_explain_reason (manager, GPM_EVENT_NOTIFICATION,
+	gpm_info_explain_reason (manager->priv->info, GPM_EVENT_NOTIFICATION,
 				_("Rebooting computer"),
 				_("the DBUS method Reboot() was invoked"));
-	return gpm_manager_reboot (manager, error);
+	return gpm_control_reboot (manager->priv->control, error);
 }
 
 /**
@@ -984,24 +576,24 @@ idle_do_sleep (GpmManager *manager)
 		gpm_debug ("doing nothing as system idle action");
 
 	} else if (strcmp (action, ACTION_SUSPEND) == 0) {
-		manager_explain_reason (manager, GPM_EVENT_SUSPEND,
+		gpm_info_explain_reason (manager->priv->info, GPM_EVENT_SUSPEND,
 					_("Suspending computer"), _("System idle"));
-		ret = gpm_manager_suspend (manager, NULL);
+		ret = gpm_control_suspend (manager->priv->control, NULL);
 		if (ret == FALSE) {
 			gpm_warning ("cannot suspend, so trying hibernate");
-			ret = gpm_manager_hibernate (manager, NULL);
+			ret = gpm_control_hibernate (manager->priv->control, NULL);
 			if (ret == FALSE) {
 				gpm_warning ("cannot suspend or hibernate!");
 			}
 		}
 
 	} else if (strcmp (action, ACTION_HIBERNATE) == 0) {
-		manager_explain_reason (manager, GPM_EVENT_HIBERNATE,
+		gpm_info_explain_reason (manager->priv->info, GPM_EVENT_HIBERNATE,
 					_("Hibernating computer"), _("System idle"));
-		ret = gpm_manager_hibernate (manager, NULL);
+		ret = gpm_control_hibernate (manager->priv->control, NULL);
 		if (ret == FALSE) {
 			gpm_warning ("cannot hibernate, so trying suspend");
-			ret = gpm_manager_suspend (manager, NULL);
+			ret = gpm_control_suspend (manager->priv->control, NULL);
 			if (ret == FALSE) {
 				gpm_warning ("cannot suspend or hibernate!");
 			}
@@ -1043,7 +635,7 @@ idle_changed_cb (GpmIdle    *idle,
 	 * support hardware blanking.
 	 * Details are here: https://launchpad.net/malone/bugs/22522
 	 */
-	if (button_is_lid_closed (manager->priv->button) == TRUE && laptop_using_ext_mon == FALSE) {
+	if (gpm_button_is_lid_closed (manager->priv->button) == TRUE && laptop_using_ext_mon == FALSE) {
 		gpm_debug ("lid is closed, so we are ignoring idle state changes");
 		return;
 	}
@@ -1059,10 +651,10 @@ idle_changed_cb (GpmIdle    *idle,
 	} else if (mode == GPM_IDLE_MODE_SYSTEM) {
 		gpm_debug ("Idle state changed: SYSTEM");
 
-		if (! gpm_manager_is_policy_timout_valid (manager, "timeout action")) {
+		if (! gpm_control_is_policy_timout_valid (manager->priv->control, "timeout action")) {
 			return;
 		}
-		if (! gpm_manager_is_inhibit_valid (manager, "timeout action")) {
+		if (! gpm_manager_is_inhibit_valid (manager, FALSE, "timeout action")) {
 			return;
 		}
 		idle_do_sleep (manager);
@@ -1100,10 +692,10 @@ battery_button_pressed (GpmManager *manager)
 static void
 power_button_pressed (GpmManager *manager)
 {
-	if (! gpm_manager_is_policy_timout_valid (manager, "power button press")) {
+	if (! gpm_control_is_policy_timout_valid (manager->priv->control, "power button press")) {
 		return;
 	}
-	if (! gpm_manager_is_inhibit_valid (manager, "power button press")) {
+	if (! gpm_manager_is_inhibit_valid (manager, TRUE, "power button press")) {
 		return;
 	}
 	gpm_debug ("power button pressed");
@@ -1119,10 +711,10 @@ power_button_pressed (GpmManager *manager)
 static void
 suspend_button_pressed (GpmManager *manager)
 {
-	if (! gpm_manager_is_policy_timout_valid (manager, "suspend button press")) {
+	if (! gpm_control_is_policy_timout_valid (manager->priv->control, "suspend button press")) {
 		return;
 	}
-	if (! gpm_manager_is_inhibit_valid (manager, "suspend button press")) {
+	if (! gpm_manager_is_inhibit_valid (manager, TRUE, "suspend button press")) {
 		return;
 	}
 	gpm_debug ("suspend button pressed");
@@ -1138,10 +730,10 @@ suspend_button_pressed (GpmManager *manager)
 static void
 hibernate_button_pressed (GpmManager *manager)
 {
-	if (! gpm_manager_is_policy_timout_valid (manager, "hibernate button press")) {
+	if (! gpm_control_is_policy_timout_valid (manager->priv->control, "hibernate button press")) {
 		return;
 	}
-	if (! gpm_manager_is_inhibit_valid (manager, "hibernate button press")) {
+	if (! gpm_manager_is_inhibit_valid (manager, TRUE, "hibernate button press")) {
 		return;
 	}
 	manager_policy_do (manager,
@@ -1261,7 +853,7 @@ ac_adapter_changed_cb (GpmAcAdapter     *ac_adapter,
 	   is closed. Fixes #331655 */
 	if (event_when_closed == TRUE &&
 	    state == GPM_AC_ADAPTER_MISSING &&
-	    button_is_lid_closed (manager->priv->button)) {
+	    gpm_button_is_lid_closed (manager->priv->button)) {
 		manager_policy_do (manager,
 				   GPM_CONF_BATTERY_BUTTON_LID,
 				   _("the lid has been closed, and the ac adapter "
@@ -1270,7 +862,7 @@ ac_adapter_changed_cb (GpmAcAdapter     *ac_adapter,
 
 	/* Don't do any events for a few seconds after we remove the
 	 * ac_adapter. See #348201 for details */
-	gpm_manager_reset_event_time (manager);
+	gpm_control_reset_event_time (manager->priv->control);
 }
 
 /**
@@ -1394,7 +986,7 @@ battery_status_changed_primary (GpmManager     *manager,
 		gchar *action;
 		timeout = GPM_NOTIFY_TIMEOUT_LONG;
 
-		if (! gpm_manager_is_policy_timout_valid (manager, "critical action")) {
+		if (! gpm_control_is_policy_timout_valid (manager->priv->control, "critical action")) {
 			return;
 		}
 
@@ -1436,13 +1028,17 @@ battery_status_changed_primary (GpmManager     *manager,
 		}
 
 	} else {
-		remaining = gpm_get_timestring (battery_status->remaining_time);
-		message = g_strdup_printf (_("You have approximately <b>%s</b> "
-					     "of remaining battery life (%d%%). "
-					     "Plug in your AC Adapter to avoid losing data."),
-					   remaining, battery_status->percentage_charge);
-		timeout = GPM_NOTIFY_TIMEOUT_LONG;
-		g_free (remaining);
+
+		gpm_conf_get_bool (manager->priv->conf, GPM_CONF_NOTIFY_LOW_POWER, &show_notify);
+		if (show_notify) {
+			remaining = gpm_get_timestring (battery_status->remaining_time);
+			message = g_strdup_printf (_("You have approximately <b>%s</b> "
+						     "of remaining battery life (%d%%). "
+						     "Plug in your AC Adapter to avoid losing data."),
+						   remaining, battery_status->percentage_charge);
+			timeout = GPM_NOTIFY_TIMEOUT_LONG;
+			g_free (remaining);
+		}
 		gpm_sound_event (manager->priv->sound, GPM_SOUND_POWER_LOW);
 	}
 
@@ -1512,7 +1108,7 @@ battery_status_changed_ups (GpmManager	   *manager,
 	if (warning_type == GPM_WARNING_ACTION) {
 		char *action;
 
-		if (! gpm_manager_is_policy_timout_valid (manager, "critical action")) {
+		if (! gpm_control_is_policy_timout_valid (manager->priv->control, "critical action")) {
 			return;
 		}
 
@@ -1711,8 +1307,7 @@ gpm_manager_class_init (GpmManagerClass *klass)
 			      NULL,
 			      g_cclosure_marshal_VOID__BOOLEAN,
 			      G_TYPE_NONE,
-			      1,
-			      G_TYPE_BOOLEAN);
+			      1, G_TYPE_BOOLEAN);
 
 	g_type_class_add_private (klass, sizeof (GpmManagerPrivate));
 }
@@ -1731,10 +1326,6 @@ conf_key_changed_cb (GpmConf     *conf,
 		   strcmp (key, GPM_CONF_AC_SLEEP_COMPUTER) == 0) {
 
 		gpm_manager_sync_policy_sleep (manager);
-
-	} else if (strcmp (key, GPM_CONF_POLICY_TIMEOUT) == 0) {
-		 gpm_conf_get_uint (manager->priv->conf, GPM_CONF_POLICY_TIMEOUT,
-		 		    &manager->priv->suppress_policy_timeout);
 	}
 }
 
@@ -1750,18 +1341,18 @@ static void
 gpm_manager_tray_icon_hibernate (GpmManager   *manager,
 				 GpmTrayIcon  *tray)
 {
-	if (! gpm_manager_is_policy_timout_valid (manager, "hibernate signal")) {
+	if (! gpm_control_is_policy_timout_valid (manager->priv->control, "hibernate signal")) {
 		return;
 	}
-	if (! gpm_manager_is_inhibit_valid (manager, "hibernate")) {
+	if (! gpm_manager_is_inhibit_valid (manager, TRUE, "hibernate")) {
 		return;
 	}
 
-	manager_explain_reason (manager,
+	gpm_info_explain_reason (manager->priv->info,
 				GPM_EVENT_HIBERNATE,
 				_("Hibernating computer"),
 				_("user clicked hibernate from tray menu"));
-	gpm_manager_hibernate (manager, NULL);
+	gpm_control_hibernate (manager->priv->control, NULL);
 }
 
 /**
@@ -1776,17 +1367,17 @@ static void
 gpm_manager_tray_icon_suspend (GpmManager   *manager,
 			       GpmTrayIcon  *tray)
 {
-	if (! gpm_manager_is_policy_timout_valid (manager, "suspend signal")) {
+	if (! gpm_control_is_policy_timout_valid (manager->priv->control, "suspend signal")) {
 		return;
 	}
-	if (! gpm_manager_is_inhibit_valid (manager, "suspend")) {
+	if (! gpm_manager_is_inhibit_valid (manager, TRUE, "suspend")) {
 		return;
 	}
-	manager_explain_reason (manager,
+	gpm_info_explain_reason (manager->priv->info,
 				GPM_EVENT_SUSPEND,
 				_("Suspending computer"),
 				_("user clicked suspend from tray menu"));
-	gpm_manager_suspend (manager, NULL);
+	gpm_control_suspend (manager->priv->control, NULL);
 }
 
 /**
@@ -1842,10 +1433,22 @@ screensaver_auth_request_cb (GpmScreensaver *screensaver,
 			     gboolean        auth_begin,
 			     GpmManager     *manager)
 {
+	GError *error;
+	gboolean ret;
 	/* only clear errors if we have finished the authentication */
 	if (auth_begin == FALSE) {
-		gpm_hal_clear_suspend_error (manager->priv->hal);
-		gpm_hal_clear_hibernate_error (manager->priv->hal);
+		error = NULL;
+		ret = gpm_hal_clear_suspend_error (manager->priv->hal, &error);
+		if (ret == FALSE) {
+			gpm_debug ("Failed to clear suspend error; %s", error->message);
+			g_error_free (error);
+		}
+		error = NULL;
+		ret = gpm_hal_clear_hibernate_error (manager->priv->hal, &error);
+		if (ret == FALSE) {
+			gpm_debug ("Failed to clear hibernate error; %s", error->message);
+			g_error_free (error);
+		}
 	}
 }
 
@@ -1860,8 +1463,8 @@ gpm_manager_at_exit (void)
 {
 	/* we can't use manager as g_atexit has no userdata */
 	GpmHal *hal = gpm_hal_new ();
-	gpm_hal_clear_suspend_error (hal);
-	gpm_hal_clear_hibernate_error (hal);
+	gpm_hal_clear_suspend_error (hal, NULL);
+	gpm_hal_clear_hibernate_error (hal, NULL);
 	g_object_unref (hal);
 }
 
@@ -1928,16 +1531,6 @@ gpm_manager_init (GpmManager *manager)
 	/* try and start an interactive service */
 	manager->priv->srv_cpufreq = gpm_srv_cpufreq_new ();
 
-	/* try and start an interactive service */
-	manager->priv->srv_dpms = gpm_srv_dpms_new ();
-	if (manager->priv->srv_dpms != NULL) {
-		/* add the new brightness lcd DBUS interface */
-		dbus_g_object_type_install_info (GPM_TYPE_SRV_DPMS,
-						 &dbus_glib_gpm_dpms_object_info);
-		dbus_g_connection_register_g_object (connection, GPM_DBUS_PATH_DPMS,
-						     G_OBJECT (manager->priv->srv_dpms));
-	}
-
 	/* try and start an interactive service */
 	manager->priv->screensaver = gpm_screensaver_new ();
 	g_signal_connect (manager->priv->screensaver, "auth-request",
@@ -1945,13 +1538,13 @@ gpm_manager_init (GpmManager *manager)
 	manager->priv->srv_screensaver = gpm_srv_screensaver_new ();
 
 	/* try an start an interactive service */
-	manager->priv->srv_brightness_lcd = gpm_srv_brightness_lcd_new ();
-	if (manager->priv->srv_brightness_lcd != NULL) {
+	manager->priv->srv_backlight = gpm_srv_backlight_new ();
+	if (manager->priv->srv_backlight != NULL) {
 		/* add the new brightness lcd DBUS interface */
-		dbus_g_object_type_install_info (GPM_TYPE_SRV_BRIGHTNESS_LCD,
-						 &dbus_glib_gpm_brightness_lcd_object_info);
-		dbus_g_connection_register_g_object (connection, GPM_DBUS_PATH_BRIGHT_LCD,
-						     G_OBJECT (manager->priv->srv_brightness_lcd));
+		dbus_g_object_type_install_info (GPM_TYPE_SRV_BACKLIGHT,
+						 &dbus_glib_gpm_backlight_object_info);
+		dbus_g_connection_register_g_object (connection, GPM_DBUS_PATH_BACKLIGHT,
+						     G_OBJECT (manager->priv->srv_backlight));
 	}
 
 	manager->priv->srv_brightness_kbd = gpm_srv_brightness_kbd_new ();
@@ -1968,13 +1561,35 @@ gpm_manager_init (GpmManager *manager)
 	manager->priv->notify = gpm_notify_new ();
 
 	/* use a class to handle the complex stuff */
+	gpm_debug ("creating new inhibit instance");
 	manager->priv->inhibit = gpm_inhibit_new ();
+	if (manager->priv->inhibit != NULL) {
+		/* add the new brightness lcd DBUS interface */
+		dbus_g_object_type_install_info (GPM_TYPE_INHIBIT,
+						 &dbus_glib_gpm_inhibit_object_info);
+		dbus_g_connection_register_g_object (connection, GPM_DBUS_PATH_INHIBIT,
+						     G_OBJECT (manager->priv->inhibit));
+	}
 
-	/* use the policy object */
-	manager->priv->policy = gpm_policy_new ();
+	/* use the control object */
+	gpm_debug ("creating new control instance");
+	manager->priv->control = gpm_control_new ();
+	if (manager->priv->control != NULL) {
+		/* add the new brightness lcd DBUS interface */
+		dbus_g_object_type_install_info (GPM_TYPE_CONTROL,
+						 &dbus_glib_gpm_control_object_info);
+		dbus_g_connection_register_g_object (connection, GPM_DBUS_PATH_CONTROL,
+						     G_OBJECT (manager->priv->control));
+	}
 
 	gpm_debug ("creating new tray icon");
 	manager->priv->tray_icon = gpm_tray_icon_new ();
+	if (manager->priv->tray_icon != NULL) {
+		dbus_g_object_type_install_info (GPM_TYPE_TRAY_ICON,
+						 &dbus_glib_gpm_ui_object_info);
+		dbus_g_connection_register_g_object (connection, GPM_DBUS_PATH_UI,
+						     G_OBJECT (manager->priv->tray_icon));
+	}
 
 	gpm_debug ("initialising info infrastructure");
 	manager->priv->info = gpm_info_new ();
@@ -1998,14 +1613,6 @@ gpm_manager_init (GpmManager *manager)
 	gpm_manager_sync_policy_sleep (manager);
 	gpm_tray_icon_sync (manager->priv->tray_icon);
 
-	gpm_conf_get_uint (manager->priv->conf, GPM_CONF_POLICY_TIMEOUT,
-			   &manager->priv->suppress_policy_timeout);
-	gpm_debug ("Using a supressed policy timeout of %i seconds",
-		   manager->priv->suppress_policy_timeout);
-
-	/* Pretend we just resumed when we start to let actions settle */
-	gpm_manager_reset_event_time (manager);
-
 	/* on startup, check if there are suspend errors left */
 	gpm_manager_check_sleep_errors (manager);
 }
@@ -2050,11 +1657,8 @@ gpm_manager_finalize (GObject *object)
 	if (manager->priv->srv_cpufreq) {
 		g_object_unref (manager->priv->srv_cpufreq);
 	}
-	if (manager->priv->srv_dpms) {
-		g_object_unref (manager->priv->srv_dpms);
-	}
-	if (manager->priv->srv_brightness_lcd) {
-		g_object_unref (manager->priv->srv_brightness_lcd);
+	if (manager->priv->srv_backlight) {
+		g_object_unref (manager->priv->srv_backlight);
 	}
 	if (manager->priv->srv_brightness_kbd) {
 		g_object_unref (manager->priv->srv_brightness_kbd);
@@ -2072,8 +1676,6 @@ GpmManager *
 gpm_manager_new (void)
 {
 	GpmManager *manager;
-
 	manager = g_object_new (GPM_TYPE_MANAGER, NULL);
-
 	return GPM_MANAGER (manager);
 }
