@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
- * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2005-2007 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -152,10 +152,10 @@ gpm_tray_icon_set_image_from_stock (GpmTrayIcon *icon,
 {
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
 
-	gpm_debug ("emitting icon-changed");
-	g_signal_emit (icon, signals [ICON_CHANGED], 0);
+	gpm_debug ("emitting icon-changed %s", stock_id);
+	g_signal_emit (icon, signals [ICON_CHANGED], 0, stock_id);
 
-	if (stock_id) {
+	if (stock_id != NULL) {
 		/* we only set a new icon if the name differs */
 		if (strcmp (icon->priv->stock_id, stock_id) != 0) {
 			gpm_debug ("Setting icon to %s", stock_id);
@@ -331,7 +331,7 @@ gpm_tray_icon_show_about_cb (GtkMenuItem *item, gpointer data)
 	gtk_show_about_dialog (NULL,
 			       "name", GPM_NAME,
 			       "version", VERSION,
-			       "copyright", "Copyright \xc2\xa9 2005-2006 Richard Hughes",
+			       "copyright", "Copyright \xc2\xa9 2005-2007 Richard Hughes",
 			       "license", license_trans,
 			       "website", GPM_HOMEPAGE_URL,
 			       "comments", GPM_DESCRIPTION,
@@ -769,10 +769,10 @@ gpm_tray_icon_sync (GpmTrayIcon *icon)
 		gpm_tray_icon_set_image_from_stock (icon, stock_id);
 		/* make sure that we are visible */
 		gpm_tray_icon_show (icon, TRUE);
-		g_free (stock_id);
 
-		gpm_debug ("emitting description-changed");
-		g_signal_emit (icon, signals [DESCRIPTION_CHANGED], 0);
+		gpm_debug ("emitting description-changed %s", stock_id);
+		g_signal_emit (icon, signals [DESCRIPTION_CHANGED], 0, stock_id);
+		g_free (stock_id);
 
 		gpm_power_get_status_summary (icon->priv->power, &tooltip, NULL);
 
